@@ -19,18 +19,27 @@ package org.apache.celeborn.plugin.flink;
 
 import java.util.HashSet;
 import java.util.Map;
+import java.util.Optional;
 import java.util.Set;
 import java.util.concurrent.CompletableFuture;
+import java.util.concurrent.ConcurrentHashMap;
+import java.util.concurrent.ExecutionException;
 import java.util.concurrent.ScheduledExecutorService;
 import java.util.concurrent.TimeUnit;
 
+import javax.annotation.Nullable;
+
+import org.apache.flink.annotation.VisibleForTesting;
 import org.apache.flink.api.common.BatchShuffleMode;
 import org.apache.flink.api.common.JobID;
 import org.apache.flink.configuration.Configuration;
 import org.apache.flink.configuration.ExecutionOptions;
 import org.apache.flink.configuration.MemorySize;
+import org.apache.flink.runtime.io.network.NettyShuffleServiceFactory;
 import org.apache.flink.runtime.io.network.partition.ResultPartitionType;
 import org.apache.flink.runtime.shuffle.JobShuffleContext;
+import org.apache.flink.runtime.shuffle.NettyShuffleDescriptor;
+import org.apache.flink.runtime.shuffle.NettyShuffleMaster;
 import org.apache.flink.runtime.shuffle.PartitionDescriptor;
 import org.apache.flink.runtime.shuffle.ProducerDescriptor;
 import org.apache.flink.runtime.shuffle.ShuffleDescriptor;
@@ -43,15 +52,21 @@ import org.slf4j.LoggerFactory;
 
 import org.apache.celeborn.client.LifecycleManager;
 import org.apache.celeborn.common.CelebornConf;
+import org.apache.celeborn.common.exception.CelebornIOException;
 import org.apache.celeborn.common.util.JavaUtils;
 import org.apache.celeborn.common.util.ThreadUtils;
+import org.apache.celeborn.plugin.flink.fallback.ShuffleFallbackPolicy;
+import org.apache.celeborn.plugin.flink.fallback.ShuffleFallbackPolicyRunner;
 import org.apache.celeborn.plugin.flink.utils.FlinkUtils;
 
-public class RemoteShuffleMaster implements ShuffleMaster<RemoteShuffleDescriptor> {
+public class RemoteShuffleMaster implements ShuffleMaster<ShuffleDescriptor> {
   private static final Logger LOG = LoggerFactory.getLogger(RemoteShuffleMaster.class);
+  private final CelebornConf conf;
   private final ShuffleMasterContext shuffleMasterContext;
   // Flink JobId -> Celeborn register shuffleIds
   private final Map<JobID, Set<Integer>> jobShuffleIds = JavaUtils.newConcurrentHashMap();
+  private final ConcurrentHashMap<JobID, String> jobFallbackPolicies =
+      JavaUtils.newConcurrentHashMap();
   private String celebornAppId;
   private volatile LifecycleManager lifecycleManager;
   private final ShuffleTaskInfo shuffleTaskInfo = new ShuffleTaskInfo();
@@ -59,51 +74,68 @@ public class RemoteShuffleMaster implements ShuffleMaster<RemoteShuffleDescripto
   private final ScheduledExecutorService executor =
       ThreadUtils.newDaemonSingleThreadScheduledExecutor(
           "celeborn-client-remote-shuffle-master-executor");
-  private final ResultPartitionAdapter resultPartitionDelegation;
   private final long lifecycleManagerTimestamp;
+  private final NettyShuffleServiceFactory nettyShuffleServiceFactory;
+  private volatile NettyShuffleMaster nettyShuffleMaster;
 
   public RemoteShuffleMaster(
-      ShuffleMasterContext shuffleMasterContext, ResultPartitionAdapter resultPartitionDelegation) {
-    checkShuffleConfig(shuffleMasterContext.getConfiguration());
+      ShuffleMasterContext shuffleMasterContext,
+      @Nullable NettyShuffleServiceFactory nettyShuffleServiceFactory) {
+    Configuration configuration = shuffleMasterContext.getConfiguration();
+    checkShuffleConfig(configuration);
+    this.conf = FlinkUtils.toCelebornConf(configuration);
     this.shuffleMasterContext = shuffleMasterContext;
-    this.resultPartitionDelegation = resultPartitionDelegation;
     this.lifecycleManagerTimestamp = System.currentTimeMillis();
+    this.nettyShuffleServiceFactory = nettyShuffleServiceFactory;
   }
 
   @Override
   public void registerJob(JobShuffleContext context) {
     JobID jobID = context.getJobId();
+    LOG.info("Register job: {}.", jobID);
     if (lifecycleManager == null) {
       synchronized (RemoteShuffleMaster.class) {
         if (lifecycleManager == null) {
           celebornAppId = FlinkUtils.toCelebornAppId(lifecycleManagerTimestamp, jobID);
           LOG.info("CelebornAppId: {}", celebornAppId);
-          CelebornConf celebornConf =
-              FlinkUtils.toCelebornConf(shuffleMasterContext.getConfiguration());
-          // if not set, set to true as default for flink
-          celebornConf.setIfMissing(CelebornConf.CLIENT_CHECKED_USE_ALLOCATED_WORKERS(), true);
-          lifecycleManager = new LifecycleManager(celebornAppId, celebornConf);
-          if (celebornConf.clientPushReplicateEnabled()) {
-            shuffleMasterContext.onFatalError(
-                new RuntimeException("Currently replicate shuffle data is unsupported for flink."));
-            return;
-          }
+          lifecycleManager = new LifecycleManager(celebornAppId, conf);
           this.shuffleResourceTracker = new ShuffleResourceTracker(executor, lifecycleManager);
         }
       }
     }
 
-    Set<Integer> previousShuffleIds = jobShuffleIds.putIfAbsent(jobID, new HashSet<>());
-    LOG.info("Register job: {}.", jobID);
-    shuffleResourceTracker.registerJob(context);
-    if (previousShuffleIds != null) {
-      throw new RuntimeException("Duplicated registration job: " + jobID);
+    lifecycleManager.applicationCount().increment();
+    try {
+      if (nettyShuffleServiceFactory != null) {
+        Optional<ShuffleFallbackPolicy> shuffleFallbackPolicy =
+            ShuffleFallbackPolicyRunner.getActivatedFallbackPolicy(context, conf, lifecycleManager);
+        if (shuffleFallbackPolicy.isPresent()) {
+          LOG.warn("Fallback to vanilla Flink NettyShuffleMaster for job: {}.", jobID);
+          String jobFallbackPolicy = shuffleFallbackPolicy.get().getClass().getName();
+          jobFallbackPolicies.put(jobID, jobFallbackPolicy);
+          lifecycleManager.computeFallbackCounts(
+              lifecycleManager.applicationFallbackCounts(), jobFallbackPolicy);
+          nettyShuffleMaster().registerJob(context);
+          return;
+        }
+      }
+      Set<Integer> previousShuffleIds = jobShuffleIds.putIfAbsent(jobID, new HashSet<>());
+      if (previousShuffleIds != null) {
+        throw new RuntimeException("Duplicated registration job: " + jobID);
+      }
+      shuffleResourceTracker.registerJob(context);
+    } catch (CelebornIOException e) {
+      throw new RuntimeException(e);
     }
   }
 
   @Override
   public void unregisterJob(JobID jobID) {
     LOG.info("Unregister job: {}.", jobID);
+    if (jobFallbackPolicies.remove(jobID) != null) {
+      nettyShuffleMaster().unregisterJob(jobID);
+      return;
+    }
     Set<Integer> shuffleIds = jobShuffleIds.remove(jobID);
     if (shuffleIds != null) {
       executor.execute(
@@ -122,46 +154,60 @@ public class RemoteShuffleMaster implements ShuffleMaster<RemoteShuffleDescripto
   }
 
   @Override
-  public CompletableFuture<RemoteShuffleDescriptor> registerPartitionWithProducer(
+  public CompletableFuture<ShuffleDescriptor> registerPartitionWithProducer(
       JobID jobID, PartitionDescriptor partitionDescriptor, ProducerDescriptor producerDescriptor) {
     return CompletableFuture.supplyAsync(
         () -> {
-          Set<Integer> shuffleIds = jobShuffleIds.get(jobID);
-          if (shuffleIds == null) {
-            throw new RuntimeException("Can not find job in lifecycleManager, job: " + jobID);
+          lifecycleManager.shuffleCount().increment();
+          String jobFallbackPolicy = jobFallbackPolicies.get(jobID);
+          if (jobFallbackPolicy != null) {
+            try {
+              lifecycleManager.computeFallbackCounts(
+                  lifecycleManager.shuffleFallbackCounts(), jobFallbackPolicy);
+              return nettyShuffleMaster()
+                  .registerPartitionWithProducer(jobID, partitionDescriptor, producerDescriptor)
+                  .get();
+            } catch (InterruptedException | ExecutionException e) {
+              throw new RuntimeException(e);
+            }
+          } else {
+            Set<Integer> shuffleIds = jobShuffleIds.get(jobID);
+            if (shuffleIds == null) {
+              throw new RuntimeException("Can not find job in lifecycleManager, job: " + jobID);
+            }
+
+            FlinkResultPartitionInfo resultPartitionInfo =
+                new FlinkResultPartitionInfo(jobID, partitionDescriptor, producerDescriptor);
+            ShuffleResourceDescriptor shuffleResourceDescriptor =
+                shuffleTaskInfo.genShuffleResourceDescriptor(
+                    resultPartitionInfo.getShuffleId(),
+                    resultPartitionInfo.getTaskId(),
+                    resultPartitionInfo.getAttemptId());
+
+            synchronized (shuffleIds) {
+              shuffleIds.add(shuffleResourceDescriptor.getShuffleId());
+            }
+
+            RemoteShuffleResource remoteShuffleResource =
+                new RemoteShuffleResource(
+                    lifecycleManager.getHost(),
+                    lifecycleManager.getPort(),
+                    lifecycleManagerTimestamp,
+                    shuffleResourceDescriptor);
+
+            shuffleResourceTracker.addPartitionResource(
+                jobID,
+                shuffleResourceDescriptor.getShuffleId(),
+                shuffleResourceDescriptor.getPartitionId(),
+                resultPartitionInfo.getResultPartitionId());
+
+            return new RemoteShuffleDescriptor(
+                celebornAppId,
+                jobID,
+                resultPartitionInfo.getShuffleId(),
+                resultPartitionInfo.getResultPartitionId(),
+                remoteShuffleResource);
           }
-
-          FlinkResultPartitionInfo resultPartitionInfo =
-              new FlinkResultPartitionInfo(jobID, partitionDescriptor, producerDescriptor);
-          ShuffleResourceDescriptor shuffleResourceDescriptor =
-              shuffleTaskInfo.genShuffleResourceDescriptor(
-                  resultPartitionInfo.getShuffleId(),
-                  resultPartitionInfo.getTaskId(),
-                  resultPartitionInfo.getAttemptId());
-
-          synchronized (shuffleIds) {
-            shuffleIds.add(shuffleResourceDescriptor.getShuffleId());
-          }
-
-          RemoteShuffleResource remoteShuffleResource =
-              new RemoteShuffleResource(
-                  lifecycleManager.getHost(),
-                  lifecycleManager.getPort(),
-                  lifecycleManagerTimestamp,
-                  shuffleResourceDescriptor);
-
-          shuffleResourceTracker.addPartitionResource(
-              jobID,
-              shuffleResourceDescriptor.getShuffleId(),
-              shuffleResourceDescriptor.getPartitionId(),
-              resultPartitionInfo.getResultPartitionId());
-
-          return new RemoteShuffleDescriptor(
-              celebornAppId,
-              jobID,
-              resultPartitionInfo.getShuffleId(),
-              resultPartitionInfo.getResultPartitionId(),
-              remoteShuffleResource);
         },
         executor);
   }
@@ -170,30 +216,34 @@ public class RemoteShuffleMaster implements ShuffleMaster<RemoteShuffleDescripto
   public void releasePartitionExternally(ShuffleDescriptor shuffleDescriptor) {
     executor.execute(
         () -> {
-          if (!(shuffleDescriptor instanceof RemoteShuffleDescriptor)) {
+          if (shuffleDescriptor instanceof RemoteShuffleDescriptor) {
+            try {
+              RemoteShuffleDescriptor descriptor = (RemoteShuffleDescriptor) shuffleDescriptor;
+              RemoteShuffleResource shuffleResource = descriptor.getShuffleResource();
+              ShuffleResourceDescriptor resourceDescriptor =
+                  shuffleResource.getMapPartitionShuffleDescriptor();
+              LOG.debug("release partition resource: {}.", resourceDescriptor);
+              lifecycleManager.releasePartition(
+                  resourceDescriptor.getShuffleId(), resourceDescriptor.getPartitionId());
+              shuffleResourceTracker.removePartitionResource(
+                  descriptor.getJobId(),
+                  resourceDescriptor.getShuffleId(),
+                  resourceDescriptor.getPartitionId());
+            } catch (Throwable throwable) {
+              // it is not a problem if we failed to release the target data partition
+              // because the session timeout mechanism will do the work for us latter
+              LOG.debug("Failed to release data partition {}.", shuffleDescriptor, throwable);
+            }
+          } else if (shuffleDescriptor instanceof NettyShuffleDescriptor) {
+            nettyShuffleMaster().releasePartitionExternally(shuffleDescriptor);
+          } else {
             LOG.error(
-                "Only RemoteShuffleDescriptor is supported {}.",
-                shuffleDescriptor.getClass().getName());
+                "Unsupported shuffle descriptor {}. Only supports {} and {}",
+                shuffleDescriptor.getClass().getName(),
+                RemoteShuffleDescriptor.class.getName(),
+                NettyShuffleDescriptor.class.getName());
             shuffleMasterContext.onFatalError(
                 new RuntimeException("Illegal shuffle descriptor type."));
-            return;
-          }
-          try {
-            RemoteShuffleDescriptor descriptor = (RemoteShuffleDescriptor) shuffleDescriptor;
-            RemoteShuffleResource shuffleResource = descriptor.getShuffleResource();
-            ShuffleResourceDescriptor resourceDescriptor =
-                shuffleResource.getMapPartitionShuffleDescriptor();
-            LOG.debug("release partition resource: {}.", resourceDescriptor);
-            lifecycleManager.releasePartition(
-                resourceDescriptor.getShuffleId(), resourceDescriptor.getPartitionId());
-            shuffleResourceTracker.removePartitionResource(
-                descriptor.getJobId(),
-                resourceDescriptor.getShuffleId(),
-                resourceDescriptor.getPartitionId());
-          } catch (Throwable throwable) {
-            // it is not a problem if we failed to release the target data partition
-            // because the session timeout mechanism will do the work for us latter
-            LOG.debug("Failed to release data partition {}.", shuffleDescriptor, throwable);
           }
         });
   }
@@ -203,8 +253,7 @@ public class RemoteShuffleMaster implements ShuffleMaster<RemoteShuffleDescripto
       TaskInputsOutputsDescriptor taskInputsOutputsDescriptor) {
     for (ResultPartitionType partitionType :
         taskInputsOutputsDescriptor.getPartitionTypes().values()) {
-      boolean isBlockingShuffle =
-          resultPartitionDelegation.isBlockingResultPartition(partitionType);
+      boolean isBlockingShuffle = partitionType.isBlockingOrBlockingPersistentResultPartition();
       if (!isBlockingShuffle) {
         throw new RuntimeException(
             "Blocking result partition type expected but found " + partitionType);
@@ -212,7 +261,6 @@ public class RemoteShuffleMaster implements ShuffleMaster<RemoteShuffleDescripto
     }
 
     int numResultPartitions = taskInputsOutputsDescriptor.getSubpartitionNums().size();
-    CelebornConf conf = FlinkUtils.toCelebornConf(shuffleMasterContext.getConfiguration());
     long numBytesPerPartition = conf.clientFlinkMemoryPerResultPartition();
     long numBytesForOutput = numBytesPerPartition * numResultPartitions;
 
@@ -231,11 +279,16 @@ public class RemoteShuffleMaster implements ShuffleMaster<RemoteShuffleDescripto
   @Override
   public void close() throws Exception {
     try {
+      jobFallbackPolicies.clear();
       jobShuffleIds.clear();
       LifecycleManager manager = lifecycleManager;
       if (null != manager) {
         manager.stop();
       }
+      if (nettyShuffleMaster != null) {
+        nettyShuffleMaster.close();
+        nettyShuffleMaster = null;
+      }
     } catch (Exception e) {
       LOG.warn("Encounter exception when shutdown: {}", e.getMessage(), e);
     }
@@ -261,4 +314,25 @@ public class RemoteShuffleMaster implements ShuffleMaster<RemoteShuffleDescripto
               BatchShuffleMode.ALL_EXCHANGES_BLOCKING.name()));
     }
   }
+
+  private NettyShuffleMaster nettyShuffleMaster() {
+    if (nettyShuffleMaster == null) {
+      synchronized (this) {
+        if (nettyShuffleMaster == null) {
+          nettyShuffleMaster = nettyShuffleServiceFactory.createShuffleMaster(shuffleMasterContext);
+        }
+      }
+    }
+    return nettyShuffleMaster;
+  }
+
+  @VisibleForTesting
+  public LifecycleManager lifecycleManager() {
+    return lifecycleManager;
+  }
+
+  @VisibleForTesting
+  public ConcurrentHashMap<JobID, String> jobFallbackPolicies() {
+    return jobFallbackPolicies;
+  }
 }
