@@ -330,10 +330,8 @@ public class RemoteShuffleResultPartitionSuiteJ {
 
     for (int subpartition = 0; subpartition < numSubpartitions; ++subpartition) {
       ByteBuffer record = EventSerializer.toSerializedEvent(EndOfPartitionEvent.INSTANCE);
-      // Flink 1.19.0
-      // [FLINK-33743][runtime] Support consuming multiple subpartitions on a single channel
       recordDataWritten(
-          record, Buffer.DataType.END_OF_PARTITION, subpartition, dataWritten, numBytesWritten);
+          record, Buffer.DataType.EVENT_BUFFER, subpartition, dataWritten, numBytesWritten);
     }
 
     outputGate
