@@ -42,6 +42,7 @@ import org.apache.flink.runtime.io.network.buffer.BufferDecompressor;
 import org.apache.flink.runtime.io.network.buffer.BufferPool;
 import org.apache.flink.runtime.io.network.partition.ResultPartitionID;
 import org.apache.flink.runtime.io.network.partition.ResultPartitionType;
+import org.apache.flink.runtime.io.network.partition.ResultSubpartitionIndexSet;
 import org.apache.flink.runtime.io.network.partition.consumer.IndexedInputGate;
 import org.apache.flink.runtime.io.network.partition.consumer.InputChannel;
 import org.apache.flink.runtime.io.network.partition.consumer.RemoteInputChannel;
@@ -95,13 +96,15 @@ public class RemoteShuffleInputGate extends AbstractRemoteShuffleInputGate {
   /** Accommodation for the incompleteness of Flink pluggable shuffle service. */
   private class FakedRemoteInputChannel extends RemoteInputChannel {
     FakedRemoteInputChannel(int channelIndex) {
+      // Flink 1.19.0
+      // [FLINK-25055][network] Support listen and notify mechanism for partition request
+      // [FLINK-33743][runtime] Support consuming multiple subpartitions on a single channel
       super(
           new SingleInputGate(
               inputGateDelegation.getTaskName(),
               inputGateDelegation.getGateIndex(),
               new IntermediateDataSetID(),
               ResultPartitionType.BLOCKING,
-              new IndexRange(0, 0),
               1,
               (a, b, c) -> {},
               () -> null,
@@ -112,7 +115,7 @@ public class RemoteShuffleInputGate extends AbstractRemoteShuffleInputGate {
               null),
           channelIndex,
           new ResultPartitionID(),
-          0,
+          new ResultSubpartitionIndexSet(new IndexRange(0, 0)),
           new ConnectionID(
               new TaskManagerLocation(ResourceID.generate(), InetAddress.getLoopbackAddress(), 1),
               0),
@@ -120,6 +123,7 @@ public class RemoteShuffleInputGate extends AbstractRemoteShuffleInputGate {
           0,
           0,
           0,
+          0,
           new SimpleCounter(),
           new SimpleCounter(),
           new FakedChannelStateWriter());
