@@ -21,11 +21,8 @@ package org.apache.celeborn.plugin.flink.tiered;
 import java.util.List;
 import java.util.concurrent.ScheduledExecutorService;
 
-import javax.annotation.Nullable;
-
 import org.apache.flink.configuration.Configuration;
 import org.apache.flink.runtime.io.disk.BatchShuffleReadBufferPool;
-import org.apache.flink.runtime.io.network.buffer.BufferCompressor;
 import org.apache.flink.runtime.io.network.partition.hybrid.tiered.common.TieredStoragePartitionId;
 import org.apache.flink.runtime.io.network.partition.hybrid.tiered.netty.TieredStorageNettyService;
 import org.apache.flink.runtime.io.network.partition.hybrid.tiered.storage.TieredStorageConsumerSpec;
@@ -103,8 +100,7 @@ public class CelebornTierFactory implements TierFactory {
       BatchShuffleReadBufferPool bufferPool,
       ScheduledExecutorService ioExecutor,
       List<TierShuffleDescriptor> shuffleDescriptors,
-      int maxRequestedBuffers,
-      @Nullable BufferCompressor bufferCompressor) {
+      int maxRequestedBuffers) {
     return new CelebornTierProducerAgent(
         conf,
         partitionId,
@@ -126,11 +122,6 @@ public class CelebornTierFactory implements TierFactory {
         conf, tieredStorageConsumerSpecs, shuffleDescriptors, bufferSizeBytes);
   }
 
-  @Override
-  public String identifier() {
-    return "celeborn";
-  }
-
   public static String getCelebornTierName() {
     return CELEBORN_TIER_NAME;
   }
