@@ -234,7 +234,12 @@ public class CelebornTierProducerAgent implements TierProducerAgent {
     try {
       if (hasRegisteredShuffle && partitionLocation != null) {
         flinkShuffleClient.mapPartitionMapperEnd(
-            shuffleId, mapId, attemptId, numPartitions, partitionLocation.getId());
+            shuffleId,
+            mapId,
+            attemptId,
+            numPartitions,
+            numSubPartitions,
+            partitionLocation.getId());
       }
     } catch (Exception e) {
       Utils.rethrowAsRuntimeException(e);
