@@ -19,10 +19,12 @@ package org.apache.celeborn.plugin.flink;
 
 import java.net.InetAddress;
 import java.net.UnknownHostException;
+import java.time.Duration;
 import java.util.Collection;
 import java.util.HashMap;
 import java.util.Map;
 import java.util.Random;
+import java.util.Set;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.ExecutionException;
 
@@ -44,6 +46,7 @@ import org.apache.flink.runtime.scheduler.strategy.ExecutionVertexID;
 import org.apache.flink.runtime.shuffle.JobShuffleContext;
 import org.apache.flink.runtime.shuffle.NettyShuffleDescriptor;
 import org.apache.flink.runtime.shuffle.PartitionDescriptor;
+import org.apache.flink.runtime.shuffle.PartitionWithMetrics;
 import org.apache.flink.runtime.shuffle.ProducerDescriptor;
 import org.apache.flink.runtime.shuffle.ShuffleDescriptor;
 import org.apache.flink.runtime.shuffle.ShuffleMasterContext;
@@ -350,6 +353,17 @@ public class RemoteShuffleMasterSuiteJ {
           Collection<ResultPartitionID> collection) {
         return CompletableFuture.completedFuture(null);
       }
+
+      @Override
+      public CompletableFuture<Collection<PartitionWithMetrics>> getPartitionWithMetrics(
+          Duration duration, Set<ResultPartitionID> set) {
+        return CompletableFuture.completedFuture(null);
+      }
+
+      @Override
+      public void notifyPartitionRecoveryStarted() {
+        // no-op
+      }
     };
   }
 
@@ -357,6 +371,8 @@ public class RemoteShuffleMasterSuiteJ {
       IntermediateDataSetID intermediateDataSetId, int partitionNum) {
     IntermediateResultPartitionID intermediateResultPartitionId =
         new IntermediateResultPartitionID(intermediateDataSetId, partitionNum);
+    // Flink 1.19.0
+    // [FLINK-33743][runtime] Align watermark at subpartition granularity
     return new PartitionDescriptor(
         intermediateDataSetId,
         10,
@@ -365,6 +381,7 @@ public class RemoteShuffleMasterSuiteJ {
         5,
         1,
         false,
+        false,
         false);
   }
 
