@@ -50,10 +50,7 @@ import org.apache.celeborn.common.identity.UserIdentifier;
 import org.apache.celeborn.common.metrics.source.Role;
 import org.apache.celeborn.common.network.TransportContext;
 import org.apache.celeborn.common.network.buffer.NettyManagedBuffer;
-import org.apache.celeborn.common.network.client.RpcResponseCallback;
-import org.apache.celeborn.common.network.client.TransportClient;
-import org.apache.celeborn.common.network.client.TransportClientBootstrap;
-import org.apache.celeborn.common.network.client.TransportClientFactory;
+import org.apache.celeborn.common.network.client.*;
 import org.apache.celeborn.common.network.protocol.*;
 import org.apache.celeborn.common.network.protocol.SerdeVersion;
 import org.apache.celeborn.common.network.sasl.SaslClientBootstrap;
@@ -122,6 +119,8 @@ public class ShuffleClientImpl extends ShuffleClient {
 
   private final boolean pushExcludeWorkerOnFailureEnabled;
   private final boolean shuffleCompressionEnabled;
+  private final boolean shuffleIntegrityCheckEnabled;
+
   private final Set<String> pushExcludedWorkers = ConcurrentHashMap.newKeySet();
   private final ConcurrentHashMap<String, Long> fetchExcludedWorkers =
       JavaUtils.newConcurrentHashMap();
@@ -203,6 +202,7 @@ public class ShuffleClientImpl extends ShuffleClient {
     shuffleCompressionEnabled = !conf.shuffleCompressionCodec().equals(CompressionCodec.NONE);
     pushReplicateEnabled = conf.clientPushReplicateEnabled();
     fetchExcludeWorkerOnFailureEnabled = conf.clientFetchExcludeWorkerOnFailureEnabled();
+    shuffleIntegrityCheckEnabled = conf.clientShuffleIntegrityCheckEnabled();
     if (conf.clientPushReplicateEnabled()) {
       pushDataTimeout = conf.pushDataTimeoutMs() * 2;
     } else {
@@ -476,7 +476,7 @@ public class ShuffleClientImpl extends ShuffleClient {
     }
 
     for (int i = index; i < reviveRequests.length; i++) {
-      ReviveRequest request = reviveRequests[index];
+      ReviveRequest request = reviveRequests[i];
       DataBatches.DataBatch batch = batches.get(i);
       if (remainReviveTimes > 0) {
         reviveFailedBatchesMap.add(batch);
@@ -648,6 +648,35 @@ public class ShuffleClientImpl extends ShuffleClient {
         });
   }
 
+  @Override
+  public void readReducerPartitionEnd(
+      int shuffleId,
+      int partitionId,
+      int startMapIndex,
+      int endMapIndex,
+      int crc32,
+      long bytesWritten)
+      throws IOException {
+    PbReadReducerPartitionEnd pbReadReducerPartitionEnd =
+        PbReadReducerPartitionEnd.newBuilder()
+            .setShuffleId(shuffleId)
+            .setPartitionId(partitionId)
+            .setStartMaxIndex(startMapIndex)
+            .setEndMapIndex(endMapIndex)
+            .setCrc32(crc32)
+            .setBytesWritten(bytesWritten)
+            .build();
+
+    PbReadReducerPartitionEndResponse pbReducerPartitionEndResponse =
+        lifecycleManagerRef.askSync(
+            pbReadReducerPartitionEnd,
+            conf.clientRpcRegisterShuffleAskTimeout(),
+            ClassTag$.MODULE$.apply(PbReadReducerPartitionEndResponse.class));
+    if (pbReducerPartitionEndResponse.getStatus() != StatusCode.SUCCESS.getValue()) {
+      throw new CelebornIOException(pbReducerPartitionEndResponse.getErrorMsg());
+    }
+  }
+
   @Override
   public boolean reportShuffleFetchFailure(int appShuffleId, int shuffleId, long taskId) {
     PbReportShuffleFetchFailure pbReportShuffleFetchFailure =
@@ -1012,6 +1041,12 @@ public class ShuffleClientImpl extends ShuffleClient {
     // increment batchId
     final int nextBatchId = pushState.nextBatchId();
 
+    // Track commit metadata if shuffle compression and integrity check are enabled and this request
+    // is not for pushing metadata itself.
+    if (shuffleIntegrityCheckEnabled) {
+      pushState.addDataWithOffsetAndLength(partitionId, data, offset, length);
+    }
+
     if (shuffleCompressionEnabled && !skipCompress) {
       // compress data
       final Compressor compressor = compressorThreadLocal.get();
@@ -1298,14 +1333,16 @@ public class ShuffleClientImpl extends ShuffleClient {
       if (shouldPush) {
         limitMaxInFlight(mapKey, pushState, loc.hostAndPushPort());
         DataBatches dataBatches = pushState.takeDataBatches(addressPair);
-        doPushMergedData(
-            addressPair,
-            shuffleId,
-            mapId,
-            attemptId,
-            dataBatches.requireBatches(),
-            pushState,
-            maxReviveTimes);
+        if (dataBatches != null) {
+          doPushMergedData(
+              addressPair,
+              shuffleId,
+              mapId,
+              attemptId,
+              dataBatches.requireBatches(),
+              pushState,
+              maxReviveTimes);
+        }
       }
     }
 
@@ -1726,19 +1763,25 @@ public class ShuffleClientImpl extends ShuffleClient {
   }
 
   @Override
-  public void mapperEnd(int shuffleId, int mapId, int attemptId, int numMappers)
+  public void mapperEnd(int shuffleId, int mapId, int attemptId, int numMappers, int numPartitions)
       throws IOException {
-    mapEndInternal(shuffleId, mapId, attemptId, numMappers, -1);
+    mapEndInternal(shuffleId, mapId, attemptId, numMappers, numPartitions, -1);
   }
 
   @Override
   public void mapPartitionMapperEnd(
-      int shuffleId, int mapId, int attemptId, int numMappers, int partitionId) throws IOException {
-    mapEndInternal(shuffleId, mapId, attemptId, numMappers, partitionId);
+      int shuffleId, int mapId, int attemptId, int numMappers, int numPartitions, int partitionId)
+      throws IOException {
+    mapEndInternal(shuffleId, mapId, attemptId, numMappers, numPartitions, partitionId);
   }
 
   private void mapEndInternal(
-      int shuffleId, int mapId, int attemptId, int numMappers, Integer partitionId)
+      int shuffleId,
+      int mapId,
+      int attemptId,
+      int numMappers,
+      int numPartitions,
+      Integer partitionId)
       throws IOException {
     final String mapKey = Utils.makeMapKey(shuffleId, mapId, attemptId);
     PushState pushState = getPushState(mapKey);
@@ -1746,6 +1789,12 @@ public class ShuffleClientImpl extends ShuffleClient {
     try {
       limitZeroInFlight(mapKey, pushState);
 
+      // send CRC32 and num bytes per partition if e2e checks are enabled
+      int[] crc32PerPartition =
+          pushState.getCRC32PerPartition(shuffleIntegrityCheckEnabled, numPartitions);
+      long[] bytesPerPartition =
+          pushState.getBytesWrittenPerPartition(shuffleIntegrityCheckEnabled, numPartitions);
+
       MapperEndResponse response =
           lifecycleManagerRef.askSync(
               new MapperEnd(
@@ -1754,7 +1803,10 @@ public class ShuffleClientImpl extends ShuffleClient {
                   attemptId,
                   numMappers,
                   partitionId,
-                  pushState.getFailedBatches()),
+                  pushState.getFailedBatches(),
+                  numPartitions,
+                  crc32PerPartition,
+                  bytesPerPartition),
               rpcMaxRetries,
               rpcRetryWait,
               ClassTag$.MODULE$.apply(MapperEndResponse.class));
