@@ -40,6 +40,7 @@ import org.apache.celeborn.client.ShuffleClient;
 import org.apache.celeborn.client.compress.Decompressor;
 import org.apache.celeborn.client.read.checkpoint.PartitionReaderCheckpointMetadata;
 import org.apache.celeborn.common.CelebornConf;
+import org.apache.celeborn.common.CommitMetadata;
 import org.apache.celeborn.common.exception.CelebornIOException;
 import org.apache.celeborn.common.network.client.TransportClient;
 import org.apache.celeborn.common.network.client.TransportClientFactory;
@@ -103,7 +104,9 @@ public abstract class CelebornInputStream extends InputStream {
             exceptionMaker,
             true,
             metricsCallback,
-            needDecompress);
+            needDecompress,
+            startMapIndex,
+            endMapIndex);
       } else {
         return new CelebornInputStreamImpl(
             conf,
@@ -126,7 +129,9 @@ public abstract class CelebornInputStream extends InputStream {
             exceptionMaker,
             false,
             metricsCallback,
-            needDecompress);
+            needDecompress,
+            -1,
+            -1);
       }
     }
   }
@@ -168,6 +173,8 @@ public abstract class CelebornInputStream extends InputStream {
     private final CelebornConf conf;
     private final TransportClientFactory clientFactory;
     private final String shuffleKey;
+    private final int numberOfSubPartitions;
+    private final int currentIndexOfSubPartition;
     private ArrayList<PartitionLocation> locations;
     private ArrayList<PbStreamHandler> streamHandlers;
     private int[] attempts;
@@ -206,6 +213,7 @@ public abstract class CelebornInputStream extends InputStream {
     private final String localHostAddress;
 
     private boolean shouldDecompress;
+    private boolean shuffleIntegrityCheckEnabled;
     private long fetchExcludedWorkerExpireTimeout;
     private ConcurrentHashMap<String, Long> fetchExcludedWorkers;
 
@@ -216,6 +224,8 @@ public abstract class CelebornInputStream extends InputStream {
     private int partitionId;
     private ExceptionMaker exceptionMaker;
     private boolean closed = false;
+    private boolean integrityChecked = false;
+    private final CommitMetadata aggregatedActualCommitMetadata = new CommitMetadata();
 
     private final boolean readSkewPartitionWithoutMapRange;
 
@@ -238,7 +248,9 @@ public abstract class CelebornInputStream extends InputStream {
         ExceptionMaker exceptionMaker,
         boolean splitSkewPartitionWithoutMapRange,
         MetricsCallback metricsCallback,
-        boolean needDecompress)
+        boolean needDecompress,
+        int numberOfSubPartitions,
+        int currentIndexOfSubPartition)
         throws IOException {
       this(
           conf,
@@ -261,7 +273,9 @@ public abstract class CelebornInputStream extends InputStream {
           exceptionMaker,
           splitSkewPartitionWithoutMapRange,
           metricsCallback,
-          needDecompress);
+          needDecompress,
+          numberOfSubPartitions,
+          currentIndexOfSubPartition);
     }
 
     CelebornInputStreamImpl(
@@ -285,7 +299,9 @@ public abstract class CelebornInputStream extends InputStream {
         ExceptionMaker exceptionMaker,
         boolean readSkewPartitionWithoutMapRange,
         MetricsCallback metricsCallback,
-        boolean needDecompress)
+        boolean needDecompress,
+        int numberOfSubPartitions,
+        int currentIndexOfSubPartition)
         throws IOException {
       this.conf = conf;
       this.clientFactory = clientFactory;
@@ -305,9 +321,12 @@ public abstract class CelebornInputStream extends InputStream {
       this.localHostAddress = Utils.localHostName(conf);
       this.shouldDecompress =
           !conf.shuffleCompressionCodec().equals(CompressionCodec.NONE) && needDecompress;
+      this.shuffleIntegrityCheckEnabled = conf.clientShuffleIntegrityCheckEnabled();
       this.fetchExcludedWorkerExpireTimeout = conf.clientFetchExcludedWorkerExpireTimeout();
       this.failedBatches = failedBatchSet;
       this.readSkewPartitionWithoutMapRange = readSkewPartitionWithoutMapRange;
+      this.numberOfSubPartitions = numberOfSubPartitions;
+      this.currentIndexOfSubPartition = currentIndexOfSubPartition;
       this.fetchExcludedWorkers = fetchExcludedWorkers;
 
       if (conf.clientPushReplicateEnabled()) {
@@ -721,6 +740,36 @@ public abstract class CelebornInputStream extends InputStream {
       }
     }
 
+    void validateIntegrity() throws IOException {
+      if (integrityChecked || !shuffleIntegrityCheckEnabled) {
+        return;
+      }
+
+      if (readSkewPartitionWithoutMapRange) {
+        shuffleClient.readReducerPartitionEnd(
+            shuffleId,
+            partitionId,
+            numberOfSubPartitions,
+            currentIndexOfSubPartition,
+            aggregatedActualCommitMetadata.getChecksum(),
+            aggregatedActualCommitMetadata.getBytes());
+      } else {
+        shuffleClient.readReducerPartitionEnd(
+            shuffleId,
+            partitionId,
+            startMapIndex,
+            endMapIndex,
+            aggregatedActualCommitMetadata.getChecksum(),
+            aggregatedActualCommitMetadata.getBytes());
+      }
+      logger.info(
+          "reducerPartitionEnd successful for shuffleId{}, partitionId{}. actual CommitMetadata: {}",
+          shuffleId,
+          partitionId,
+          aggregatedActualCommitMetadata);
+      integrityChecked = true;
+    }
+
     private boolean moveToNextChunk() throws IOException {
       if (currentChunk != null) {
         currentChunk.release();
@@ -760,6 +809,7 @@ public abstract class CelebornInputStream extends InputStream {
           firstChunk = false;
         }
         if (currentChunk == null) {
+          validateIntegrity();
           close();
           return false;
         }
@@ -821,6 +871,9 @@ public abstract class CelebornInputStream extends InputStream {
               } else {
                 limit = size;
               }
+              if (shuffleIntegrityCheckEnabled) {
+                aggregatedActualCommitMetadata.addDataWithOffsetAndLength(rawDataBuf, 0, limit);
+              }
               position = 0;
               hasData = true;
               break;
@@ -835,6 +888,10 @@ public abstract class CelebornInputStream extends InputStream {
           }
         }
 
+        if (!hasData) {
+          validateIntegrity();
+          // TODO(gaurav): consider closing the stream
+        }
         return hasData;
       } catch (LZ4Exception | ZstdException | IOException e) {
         logger.error(
