@@ -60,7 +60,7 @@ public class DfsPartitionReader implements PartitionReader {
   private final long shuffleChunkSize;
   private final int fetchMaxReqsInFlight;
   private final LinkedBlockingQueue<Pair<Integer, ByteBuf>> results;
-  private final AtomicReference<IOException> exception = new AtomicReference<>();
+  private final AtomicReference<Exception> exception = new AtomicReference<>();
   private volatile boolean closed = false;
   private ExecutorService fetchThread;
   private boolean fetchThreadStarted;
@@ -75,6 +75,7 @@ public class DfsPartitionReader implements PartitionReader {
   private TransportClient client;
   private PbStreamHandler streamHandler;
   private MetricsCallback metricsCallback;
+  private long partitionReaderWaitLogThreshold;
   private FileSystem hadoopFs;
 
   private Path dataFilePath;
@@ -100,6 +101,7 @@ public class DfsPartitionReader implements PartitionReader {
     results = new LinkedBlockingQueue<>();
 
     this.metricsCallback = metricsCallback;
+    this.partitionReaderWaitLogThreshold = conf.clientPartitionReaderWaitLogThreshold();
     this.location = location;
     if (location.getStorageInfo() != null
         && location.getStorageInfo().getType() == StorageInfo.Type.S3) {
@@ -236,7 +238,7 @@ public class DfsPartitionReader implements PartitionReader {
   }
 
   @Override
-  public ByteBuf next() throws IOException, InterruptedException {
+  public ByteBuf next() throws Exception {
     Pair<Integer, ByteBuf> chunk = null;
     checkpoint();
     if (!fetchThreadStarted) {
@@ -283,21 +285,44 @@ public class DfsPartitionReader implements PartitionReader {
               }
             } catch (Exception e) {
               logger.warn("Fetch thread is cancelled.", e);
+              exception.set(e);
               // cancel a task for speculative, ignore this exception
             }
             logger.debug("fetch {} is done.", location.getStorageInfo().getFilePath());
           });
     }
     try {
+      long totalWaitTimeMs = 0;
+      long lastLogTimeMs = 0;
+
       while (chunk == null) {
         checkException();
         Long startFetchWait = System.nanoTime();
         chunk = results.poll(500, TimeUnit.MILLISECONDS);
-        metricsCallback.incReadTime(
-            TimeUnit.NANOSECONDS.toMillis(System.nanoTime() - startFetchWait));
+        long waitTimeMs = TimeUnit.NANOSECONDS.toMillis(System.nanoTime() - startFetchWait);
+        metricsCallback.incReadTime(waitTimeMs);
+        totalWaitTimeMs += waitTimeMs;
+        // Log when wait time exceeds another threshold since last log
+        if (chunk == null && totalWaitTimeMs >= lastLogTimeMs + partitionReaderWaitLogThreshold) {
+          lastLogTimeMs = totalWaitTimeMs;
+          logger.info(
+              "Waiting for data from partition {}/{} for {}ms",
+              location.getFileName(),
+              location.hostAndPorts(),
+              totalWaitTimeMs);
+        }
+
         logger.debug("poll result with result size: {}", results.size());
       }
-    } catch (InterruptedException e) {
+
+      if (totalWaitTimeMs >= partitionReaderWaitLogThreshold) {
+        logger.info(
+            "Finished waiting for data from partition {}/{} after {}ms",
+            location.getFileName(),
+            location.hostAndPorts(),
+            totalWaitTimeMs);
+      }
+    } catch (Exception e) {
       logger.error("PartitionReader thread interrupted while fetching data.");
       throw e;
     }
@@ -306,8 +331,8 @@ public class DfsPartitionReader implements PartitionReader {
     return chunk.getRight();
   }
 
-  private void checkException() throws IOException {
-    IOException e = exception.get();
+  private void checkException() throws Exception {
+    Exception e = exception.get();
     if (e != null) {
       throw e;
     }
