@@ -24,8 +24,6 @@ import java.nio.ByteBuffer;
 import io.netty.buffer.ByteBuf;
 import io.netty.buffer.ByteBufInputStream;
 import io.netty.buffer.Unpooled;
-import org.apache.commons.lang3.builder.ToStringBuilder;
-import org.apache.commons.lang3.builder.ToStringStyle;
 
 /** A {@link ManagedBuffer} backed by a Netty {@link ByteBuf}. */
 public class NettyManagedBuffer extends ManagedBuffer {
@@ -83,8 +81,6 @@ public class NettyManagedBuffer extends ManagedBuffer {
 
   @Override
   public String toString() {
-    return new ToStringBuilder(this, ToStringStyle.SHORT_PREFIX_STYLE)
-        .append("buf", buf)
-        .toString();
+    return "NettyManagedBuffer[buf=" + buf + "]";
   }
 }
