@@ -23,8 +23,6 @@ import java.nio.ByteBuffer;
 
 import io.netty.buffer.ByteBufInputStream;
 import io.netty.buffer.Unpooled;
-import org.apache.commons.lang3.builder.ToStringBuilder;
-import org.apache.commons.lang3.builder.ToStringStyle;
 
 /** A {@link ManagedBuffer} backed by {@link ByteBuffer}. */
 public class NioManagedBuffer extends ManagedBuffer {
@@ -71,8 +69,6 @@ public class NioManagedBuffer extends ManagedBuffer {
 
   @Override
   public String toString() {
-    return new ToStringBuilder(this, ToStringStyle.SHORT_PREFIX_STYLE)
-        .append("buf", buf)
-        .toString();
+    return "NioManagedBuffer[buf=" + buf + "]";
   }
 }
