@@ -92,7 +92,8 @@ public enum StatusCode {
   SEGMENT_START_FAIL_REPLICA(52),
   SEGMENT_START_FAIL_PRIMARY(53),
   NO_SPLIT(54),
-  WORKER_UNRESPONSIVE(55);
+  WORKER_UNRESPONSIVE(55),
+  READ_REDUCER_PARTITION_END_FAILED(56);
 
   private final byte value;
 
