@@ -20,8 +20,8 @@ package org.apache.celeborn.common
 import org.apache.celeborn.CelebornFunSuite
 import org.apache.celeborn.common.CelebornConf._
 import org.apache.celeborn.common.internal.config.ConfigEntry
+import org.apache.celeborn.common.network.util.IOMode
 import org.apache.celeborn.common.protocol.StorageInfo
-import org.apache.celeborn.common.util.Utils
 
 class CelebornConfSuite extends CelebornFunSuite {
 
@@ -195,26 +195,41 @@ class CelebornConfSuite extends CelebornFunSuite {
 
   test("Test empty working dir") {
     val conf = new CelebornConf()
-    conf.set("celeborn.storage.activeTypes", "HDFS")
+    conf.set("celeborn.storage.availableTypes", "HDFS")
     conf.set("celeborn.storage.hdfs.dir", "hdfs:///xxx")
     assert(conf.workerBaseDirs.isEmpty)
 
-    conf.set("celeborn.storage.activeTypes", "SSD,HDD,HDFS")
+    conf.set("celeborn.storage.availableTypes", "SSD,HDD,HDFS")
     conf.set("celeborn.storage.hdfs.dir", "hdfs:///xxx")
     assert(conf.workerBaseDirs.isEmpty)
 
-    conf.set("celeborn.storage.activeTypes", "SSD,HDD")
+    conf.set("celeborn.storage.availableTypes", "SSD,HDD")
     assert(!conf.workerBaseDirs.isEmpty)
+
+    conf.set("celeborn.storage.availableTypes", "S3")
+    conf.set("celeborn.storage.s3.dir", "s3a:///xxx")
+    assert(conf.workerBaseDirs.isEmpty)
   }
 
   test("Test commit file threads") {
     val conf = new CelebornConf()
-    conf.set("celeborn.storage.activeTypes", "HDFS")
+    conf.set("celeborn.storage.availableTypes", "HDFS")
     conf.set("celeborn.storage.hdfs.dir", "hdfs:///xxx")
     assert(conf.workerCommitThreads === 128)
 
-    conf.set("celeborn.storage.activeTypes", "SSD,HDD")
+    conf.set("celeborn.storage.availableTypes", "SSD,HDD")
     assert(conf.workerCommitThreads === 32)
+
+    conf.set("celeborn.storage.availableTypes", "S3")
+    conf.set("celeborn.storage.s3.dir", "s3a:///xxx")
+    assert(conf.workerCommitThreads === 32)
+  }
+
+  test("Test commit buffer size") {
+    val conf = new CelebornConf()
+    conf.set("celeborn.storage.availableTypes", "S3")
+    conf.set("celeborn.storage.s3.dir", "s3a:///xxx")
+    assert(conf.workerS3FlusherBufferSize === 6291456)
   }
 
   test("Test available storage types") {
@@ -230,6 +245,12 @@ class CelebornConfSuite extends CelebornFunSuite {
 
     conf.set("celeborn.storage.availableTypes", "HDFS")
     assert(conf.availableStorageTypes == StorageInfo.HDFS_MASK)
+
+    conf.set("celeborn.storage.availableTypes", "S3")
+    assert(conf.availableStorageTypes == StorageInfo.S3_MASK)
+
+    conf.set("celeborn.storage.availableTypes", "OSS")
+    assert(conf.availableStorageTypes == StorageInfo.OSS_MASK)
   }
 
   test("Test role rpcDispatcherNumThreads") {
@@ -257,7 +278,7 @@ class CelebornConfSuite extends CelebornFunSuite {
 
   // Transport conf tests
 
-  private val transportTestNetworkIoMode = "EPOLL"
+  private val transportTestNetworkIoMode = IOMode.EPOLL.name()
   private val transportTestNetworkIoPreferDirectBufs =
     !NETWORK_IO_PREFER_DIRECT_BUFS.defaultValue.get
 
@@ -275,7 +296,6 @@ class CelebornConfSuite extends CelebornFunSuite {
   private val transportTestNetworkIoStorageMemoryMapThreshold =
     NETWORK_IO_STORAGE_MEMORY_MAP_THRESHOLD.defaultValue.get + 5
   private val transportTestNetworkIoLazyFd = !NETWORK_IO_LAZY_FD.defaultValue.get
-  private val transportTestNetworkVerboseMetrics = !NETWORK_VERBOSE_METRICS.defaultValue.get
   private val transportTestChannelHeartbeatInterval =
     CHANNEL_HEARTBEAT_INTERVAL.defaultValue.get + 5
   private val transportTestPushTimeoutCheckThreads = PUSH_TIMEOUT_CHECK_THREADS.defaultValue.get + 5
@@ -293,6 +313,7 @@ class CelebornConfSuite extends CelebornFunSuite {
     def moduleKey(config: ConfigEntry[_]): String = {
       config.key.replace("<module>", module)
     }
+
     conf.set(moduleKey(NETWORK_IO_MODE), transportTestNetworkIoMode)
     conf.set(
       moduleKey(NETWORK_IO_PREFER_DIRECT_BUFS),
@@ -315,7 +336,6 @@ class CelebornConfSuite extends CelebornFunSuite {
       moduleKey(NETWORK_IO_STORAGE_MEMORY_MAP_THRESHOLD),
       transportTestNetworkIoStorageMemoryMapThreshold.toString)
     conf.set(moduleKey(NETWORK_IO_LAZY_FD), transportTestNetworkIoLazyFd.toString)
-    conf.set(moduleKey(NETWORK_VERBOSE_METRICS), transportTestNetworkVerboseMetrics.toString)
     conf.set(moduleKey(CHANNEL_HEARTBEAT_INTERVAL), transportTestChannelHeartbeatInterval.toString)
     conf.set(moduleKey(PUSH_TIMEOUT_CHECK_THREADS), transportTestPushTimeoutCheckThreads.toString)
     conf.set(moduleKey(PUSH_TIMEOUT_CHECK_INTERVAL), transportTestPushTimeoutCheckInterval.toString)
@@ -328,7 +348,7 @@ class CelebornConfSuite extends CelebornFunSuite {
     conf
   }
 
-  private def validateDefauitTransportConfValue(conf: CelebornConf, module: String): Unit = {
+  private def validateDefaultTransportConfValue(conf: CelebornConf, module: String): Unit = {
     assert(transportTestNetworkIoMode == conf.networkIoMode(module))
     assert(transportTestNetworkIoPreferDirectBufs == conf.networkIoPreferDirectBufs(module))
     assert(transportTestNetworkIoConnectTimeout == conf.networkIoConnectTimeoutMs(module))
@@ -344,8 +364,7 @@ class CelebornConfSuite extends CelebornFunSuite {
     assert(transportTestNetworkIoRetryWait == conf.networkIoRetryWaitMs(module))
     assert(transportTestNetworkIoStorageMemoryMapThreshold == conf.networkIoMemoryMapBytes(module))
     assert(transportTestNetworkIoLazyFd == conf.networkIoLazyFileDescriptor(module))
-    assert(transportTestNetworkVerboseMetrics == conf.networkIoVerboseMetrics(module))
-    assert(transportTestChannelHeartbeatInterval == conf.clientHeartbeatInterval(module))
+    assert(transportTestChannelHeartbeatInterval == conf.channelHeartbeatInterval(module))
     assert(transportTestPushTimeoutCheckThreads == conf.pushDataTimeoutCheckerThreads(module))
     assert(transportTestPushTimeoutCheckInterval == conf.pushDataTimeoutCheckInterval(module))
     assert(transportTestFetchTimeoutCheckThreads == conf.fetchDataTimeoutCheckerThreads(module))
@@ -355,20 +374,20 @@ class CelebornConfSuite extends CelebornFunSuite {
 
   test("Basic fetch module config") {
     val conf = setupCelebornConfForTransportTests("test")
-    validateDefauitTransportConfValue(conf, "test")
+    validateDefaultTransportConfValue(conf, "test")
   }
 
   test("Fallback to parent module's config for transport conf when not defined for module") {
     val conf = setupCelebornConfForTransportTests("test_parent_module")
     // set in parent, but should work in child
-    validateDefauitTransportConfValue(conf, "test_child_module")
+    validateDefaultTransportConfValue(conf, "test_child_module")
   }
 
   test("rpc_service and rpc_client should default to rpc if not configured") {
     val conf = setupCelebornConfForTransportTests("rpc")
     // set in rpc, so should work for specific rpc servers
-    validateDefauitTransportConfValue(conf, "rpc_service")
-    validateDefauitTransportConfValue(conf, "rpc_app")
+    validateDefaultTransportConfValue(conf, "rpc_service")
+    validateDefaultTransportConfValue(conf, "rpc_app")
   }
 
   test("Test fallback config works even with parent") {
@@ -394,6 +413,57 @@ class CelebornConfSuite extends CelebornFunSuite {
     assert(conf.networkIoConnectTimeoutMs("test_child_module") == fallbackValue)
   }
 
+  test("Test storage policy case 1") {
+    val conf = new CelebornConf()
+
+    conf.set("celeborn.worker.storage.storagePolicy.createFilePolicy", "MEMORY,S3")
+    val createFilePolicy = conf.workerStoragePolicyCreateFilePolicy
+    assert(List("MEMORY", "S3") == createFilePolicy.get)
+
+    conf.set("celeborn.worker.storage.storagePolicy.createFilePolicy", "MEMORY,SSD")
+    val createFilePolicy1 = conf.workerStoragePolicyCreateFilePolicy
+    assert(List("MEMORY", "SSD") == createFilePolicy1.get)
+
+    conf.set("celeborn.worker.storage.storagePolicy.createFilePolicy", "MEMORY,HDFS")
+    val createFilePolicy2 = conf.workerStoragePolicyCreateFilePolicy
+    assert(List("MEMORY", "HDFS") == createFilePolicy2.get)
+
+    conf.unset("celeborn.worker.storage.storagePolicy.createFilePolicy")
+    val createFilePolicy3 = conf.workerStoragePolicyCreateFilePolicy
+    assert(List("MEMORY", "SSD", "HDD", "HDFS", "S3", "OSS") == createFilePolicy3.get)
+
+    try {
+      conf.set("celeborn.worker.storage.storagePolicy.createFilePolicy", "ABC")
+      val createFilePolicy4 = conf.workerStoragePolicyCreateFilePolicy
+    } catch {
+      case e: Exception =>
+        assert(e.isInstanceOf[IllegalArgumentException])
+    }
+  }
+
+  test("Test storage policy case 2") {
+    val conf = new CelebornConf()
+    conf.set("celeborn.worker.storage.storagePolicy.evictPolicy", "MEMORY,SSD")
+    val evictPolicy1 = conf.workerStoragePolicyEvictFilePolicy
+    assert(Map("MEMORY" -> List("SSD")) == evictPolicy1.get)
+
+    conf.set("celeborn.worker.storage.storagePolicy.evictPolicy", "MEMORY,SSD,HDFS|HDD,HDFS")
+    val evictPolicy2 = conf.workerStoragePolicyEvictFilePolicy
+    assert(Map("MEMORY" -> List("SSD", "HDFS"), "HDD" -> List("HDFS")) == evictPolicy2.get)
+
+    conf.unset("celeborn.worker.storage.storagePolicy.evictPolicy")
+    val evictPolicy3 = conf.workerStoragePolicyEvictFilePolicy
+    assert(Map("MEMORY" -> List("SSD", "HDD", "HDFS", "S3", "OSS")) == evictPolicy3.get)
+
+    try {
+      conf.set("celeborn.worker.storage.storagePolicy.evictPolicy", "ABC")
+      conf.workerStoragePolicyEvictFilePolicy
+    } catch {
+      case e: Exception =>
+        assert(e.isInstanceOf[IllegalArgumentException])
+    }
+  }
+
   test("CELEBORN-1511: Test master endpoint resolver") {
     val conf = new CelebornConf()
 
