@@ -20,6 +20,9 @@ package org.apache.celeborn.common.util
 import java.util
 import java.util.Collections
 
+import org.scalatest.matchers.must.Matchers.contain
+import org.scalatest.matchers.should.Matchers.convertToAnyShouldWrapper
+
 import org.apache.celeborn.CelebornFunSuite
 import org.apache.celeborn.common.CelebornConf
 import org.apache.celeborn.common.client.{MasterEndpointResolver, StaticMasterEndpointResolver}
@@ -145,10 +148,19 @@ class UtilsSuite extends CelebornFunSuite {
   }
 
   test("MapperEnd class convert with pb") {
-    val mapperEnd = MapperEnd(1, 1, 1, 2, 1, Collections.emptyMap())
+    val mapperEnd =
+      MapperEnd(1, 1, 1, 2, 1, Collections.emptyMap(), 1, Array.emptyIntArray, Array.emptyLongArray)
     val mapperEndTrans =
       Utils.fromTransportMessage(Utils.toTransportMessage(mapperEnd)).asInstanceOf[MapperEnd]
-    assert(mapperEnd == mapperEndTrans)
+    assert(mapperEnd.shuffleId == mapperEndTrans.shuffleId)
+    assert(mapperEnd.mapId == mapperEndTrans.mapId)
+    assert(mapperEnd.attemptId == mapperEndTrans.attemptId)
+    assert(mapperEnd.numMappers == mapperEndTrans.numMappers)
+    assert(mapperEnd.partitionId == mapperEndTrans.partitionId)
+    assert(mapperEnd.failedBatchSet == mapperEndTrans.failedBatchSet)
+    assert(mapperEnd.numPartitions == mapperEndTrans.numPartitions)
+    mapperEnd.crc32PerPartition.array should contain theSameElementsInOrderAs mapperEndTrans.crc32PerPartition
+    mapperEnd.bytesWrittenPerPartition.array should contain theSameElementsInOrderAs mapperEndTrans.bytesWrittenPerPartition
   }
 
   test("validate HDFS compatible fs path") {
