@@ -28,6 +28,9 @@ import org.apache.celeborn.common.meta.WorkerStatus;
 import org.apache.celeborn.common.quota.ResourceConsumption;
 
 public interface IMetadataHandler {
+  void handleRegisterApplicationInfo(
+      String appId, UserIdentifier userIdentifier, Map<String, String> extraInfo, String requestId);
+
   void handleRequestSlots(
       String shuffleKey,
       String hostName,
