@@ -50,6 +50,15 @@ public class SingleMasterMetaManager extends AbstractMetaManager {
     this.rackResolver = rackResolver;
   }
 
+  @Override
+  public void handleRegisterApplicationInfo(
+      String appId,
+      UserIdentifier userIdentifier,
+      Map<String, String> extraInfo,
+      String requestId) {
+    updateApplicationInfo(appId, userIdentifier, extraInfo);
+  }
+
   @Override
   public void handleRequestSlots(
       String shuffleKey,
