@@ -19,14 +19,12 @@ package org.apache.celeborn.service.deploy.master.clustermeta.ha;
 
 import java.io.File;
 import java.io.IOException;
-import java.net.Inet6Address;
 import java.net.InetAddress;
 import java.net.InetSocketAddress;
 import java.nio.charset.StandardCharsets;
 import java.util.*;
 import java.util.concurrent.ScheduledExecutorService;
 import java.util.concurrent.TimeUnit;
-import java.util.concurrent.atomic.AtomicLong;
 import java.util.concurrent.locks.ReentrantReadWriteLock;
 
 import javax.net.ssl.KeyManager;
@@ -35,7 +33,6 @@ import javax.net.ssl.TrustManager;
 import scala.Tuple2;
 
 import com.google.common.annotations.VisibleForTesting;
-import com.google.common.net.InetAddresses;
 import com.google.protobuf.InvalidProtocolBufferException;
 import org.apache.ratis.RaftConfigKeys;
 import org.apache.ratis.client.RaftClientConfigKeys;
@@ -78,11 +75,7 @@ public class HARaftServer {
 
   public static final RaftGroupId RAFT_GROUP_ID = RaftGroupId.valueOf(CELEBORN_UUID);
 
-  private static final AtomicLong CALL_ID_COUNTER = new AtomicLong();
-
-  static long nextCallId() {
-    return CALL_ID_COUNTER.getAndIncrement() & Long.MAX_VALUE;
-  }
+  public static final long REQUEST_TIMEOUT_MS = 60_000;
 
   private final MasterNode localNode;
   private final InetSocketAddress ratisAddr;
@@ -174,22 +167,6 @@ public class HARaftServer {
         TimeUnit.MILLISECONDS);
   }
 
-  // copy of org.apache.ratis.util.NetUtils.address2String
-  private static String address2String(InetSocketAddress address) {
-    if (address == null) {
-      return null;
-    }
-    String hostName = address.getHostName();
-    final StringBuilder b = new StringBuilder(hostName);
-    // Surround with '[', ']' only if it is a IPv6 ip - not for a IPv6 host
-    if (address.getAddress() instanceof Inet6Address
-        && InetAddresses.isInetAddress(hostName)
-        && InetAddresses.forString(hostName).getAddress().length == 16) {
-      b.insert(0, '[').append(']');
-    }
-    return b.append(':').append(address.getPort()).toString();
-  }
-
   public static HARaftServer newMasterRatisServer(
       MetaHandler metaHandler, CelebornConf conf, MasterNode localNode, List<MasterNode> peerNodes)
       throws IOException {
@@ -200,9 +177,7 @@ public class HARaftServer {
     RaftPeer localRaftPeer =
         RaftPeer.newBuilder()
             .setId(localRaftPeerId)
-            // TODO(CELEBORN-1525): Once Ratis 3.2.0 is released and we move to it, we
-            // should remove address2String and change this to .setAddress(ratisAddr)
-            .setAddress(address2String(ratisAddr))
+            .setAddress(ratisAddr)
             .setClientAddress(localNode.rpcEndpoint())
             // We use admin address to host the internal rpc address
             .setAdminAddress(localNode.internalRpcEndpoint())
@@ -229,7 +204,7 @@ public class HARaftServer {
             raftPeer =
                 RaftPeer.newBuilder()
                     .setId(raftPeerId)
-                    .setAddress(address2String(peerRatisAddr))
+                    .setAddress(peerRatisAddr)
                     .setClientAddress(peer.rpcEndpoint())
                     // We use admin address to host the internal rpc address
                     .setAdminAddress(peer.internalRpcEndpoint())
@@ -359,7 +334,10 @@ public class HARaftServer {
     RaftServerConfigKeys.Log.setPurgeGap(properties, logPurgeGap);
 
     // For grpc set the maximum message size
-    GrpcConfigKeys.setMessageSizeMax(properties, SizeInBytes.valueOf(logAppenderQueueByteLimit));
+    // RATIS-2135. The leader keeps sending inconsistent entries repeatedly to followers.
+    // raft.grpc.message.size.max must be 1m larger than raft.server.log.appender.buffer.byte-limit
+    GrpcConfigKeys.setMessageSizeMax(
+        properties, SizeInBytes.valueOf(logAppenderQueueByteLimit + SizeInBytes.ONE_MB.getSize()));
 
     // Set the server request timeout
     TimeDuration serverRequestTimeout =
@@ -386,6 +364,10 @@ public class HARaftServer {
     RaftServerConfigKeys.Rpc.setFirstElectionTimeoutMin(properties, firstElectionTimeoutMin);
     RaftServerConfigKeys.Rpc.setFirstElectionTimeoutMax(properties, firstElectionTimeoutMax);
 
+    boolean leaderElectionMemberMajorityAdd = conf.hasMasterRatisLeaderElectionMemberMajorityAdd();
+    RaftServerConfigKeys.LeaderElection.setMemberMajorityAdd(
+        properties, leaderElectionMemberMajorityAdd);
+
     // Set the rpc client timeout
     TimeDuration clientRpcTimeout =
         TimeDuration.valueOf(conf.haMasterRatisClientRpcTimeout(), TimeUnit.SECONDS);
@@ -513,7 +495,7 @@ public class HARaftServer {
   /**
    * Get the suggested leader peer id.
    *
-   * @return RaftPeerId of the suggested leader node - Optional<LeaderPeerEndpoints>
+   * @return RaftPeerId of the suggested leader node - {@code Optional<LeaderPeerEndpoints>}
    */
   public Optional<LeaderPeerEndpoints> getCachedLeaderPeerRpcEndpoint() {
     this.roleCheckLock.readLock().lock();
@@ -613,7 +595,7 @@ public class HARaftServer {
 
   public GroupInfoReply getGroupInfo() throws IOException {
     GroupInfoRequest groupInfoRequest =
-        new GroupInfoRequest(clientId, raftPeerId, RAFT_GROUP_ID, nextCallId());
+        new GroupInfoRequest(clientId, raftPeerId, RAFT_GROUP_ID, CallId.getAndIncrement());
     return server.getGroupInfo(groupInfoRequest);
   }
 
@@ -643,7 +625,7 @@ public class HARaftServer {
               raftGroup.getGroupId(),
               CallId.getAndIncrement(),
               null,
-              60_000);
+              REQUEST_TIMEOUT_MS);
       RaftClientReply reply = server.transferLeadership(request);
       if (reply.isSuccess()) {
         LOG.info("Successfully step down leader {}.", server.getId());
@@ -668,11 +650,22 @@ public class HARaftServer {
     return appTimeoutDeadline;
   }
 
-  @VisibleForTesting
+  public ClientId getClientId() {
+    return clientId;
+  }
+
   public RaftServer getServer() {
     return server;
   }
 
+  public RaftGroupId getGroupId() {
+    return raftGroup.getGroupId();
+  }
+
+  public String getLocalAddress() {
+    return localNode.ratisEndpoint();
+  }
+
   public static class LeaderPeerEndpoints {
     // the rpcEndpoints Tuple2 (ip:port, host:port)
     public final Tuple2<String, String> rpcEndpoints;
