@@ -38,7 +38,6 @@ import org.mockito.Mockito;
 import org.apache.celeborn.common.CelebornConf;
 import org.apache.celeborn.common.client.MasterClient;
 import org.apache.celeborn.common.identity.UserIdentifier;
-import org.apache.celeborn.common.meta.AppDiskUsageSnapShot;
 import org.apache.celeborn.common.meta.DiskInfo;
 import org.apache.celeborn.common.meta.WorkerInfo;
 import org.apache.celeborn.common.quota.ResourceConsumption;
@@ -205,62 +204,41 @@ public class MasterStateMachineSuiteJ extends RatisBaseSuiteJ {
     String host2 = "host2";
     String host3 = "host3";
 
-    masterStatusSystem.excludedWorkers.add(info1);
-    masterStatusSystem.excludedWorkers.add(info2);
-    masterStatusSystem.excludedWorkers.add(info3);
+    masterStatusSystem.updateExcludedWorkersMeta(
+        Arrays.asList(info1, info2, info3), Collections.emptyList());
 
-    masterStatusSystem.manuallyExcludedWorkers.add(info1);
-    masterStatusSystem.manuallyExcludedWorkers.add(info2);
+    masterStatusSystem.updateManuallyExcludedWorkersMeta(
+        Arrays.asList(info1, info2), Collections.emptyList());
 
     masterStatusSystem.hostnameSet.add(host1);
     masterStatusSystem.hostnameSet.add(host2);
     masterStatusSystem.hostnameSet.add(host3);
 
-    // Wait for update snapshot
-    Thread.sleep(60000);
-    Map<String, Long> appDiskUsage = JavaUtils.newConcurrentHashMap();
-    appDiskUsage.put("app-1", 100L);
-    appDiskUsage.put("app-2", 200L);
-    masterStatusSystem.appDiskUsageMetric.update(appDiskUsage);
-    appDiskUsage.put("app-3", 300L);
-    appDiskUsage.put("app-1", 200L);
-    masterStatusSystem.appDiskUsageMetric.update(appDiskUsage);
-    // wait for snapshot updated
-    Thread.sleep(3000);
-
-    AppDiskUsageSnapShot[] originSnapshots = masterStatusSystem.appDiskUsageMetric.snapShots();
-    AppDiskUsageSnapShot originCurrentSnapshot =
-        masterStatusSystem.appDiskUsageMetric.currentSnapShot().get();
-
-    masterStatusSystem.workers.add(new WorkerInfo(host1, 9095, 9094, 9093, 9092, 9091));
-    masterStatusSystem.workers.add(new WorkerInfo(host2, 9095, 9094, 9093, 9092, 9091));
-    masterStatusSystem.workers.add(new WorkerInfo(host3, 9095, 9094, 9093, 9092, 9091));
+    WorkerInfo workerInfo1 = new WorkerInfo(host1, 9095, 9094, 9093, 9092, 9091);
+    WorkerInfo workerInfo2 = new WorkerInfo(host2, 9095, 9094, 9093, 9092, 9091);
+    WorkerInfo workerInfo3 = new WorkerInfo(host3, 9095, 9094, 9093, 9092, 9091);
+    masterStatusSystem.workersMap.put(workerInfo1.toUniqueId(), workerInfo1);
+    masterStatusSystem.workersMap.put(workerInfo2.toUniqueId(), workerInfo2);
+    masterStatusSystem.workersMap.put(workerInfo3.toUniqueId(), workerInfo3);
 
     masterStatusSystem.writeMetaInfoToFile(tmpFile);
 
     masterStatusSystem.hostnameSet.clear();
     masterStatusSystem.excludedWorkers.clear();
     masterStatusSystem.manuallyExcludedWorkers.clear();
-    masterStatusSystem.workers.clear();
+    masterStatusSystem.workersMap.clear();
+    masterStatusSystem.availableWorkers.clear();
 
     masterStatusSystem.restoreMetaFromFile(tmpFile);
 
-    Assert.assertEquals(3, masterStatusSystem.workers.size());
+    Assert.assertEquals(3, masterStatusSystem.workersMap.size());
+    Assert.assertEquals(3, masterStatusSystem.availableWorkers.size());
     Assert.assertEquals(3, masterStatusSystem.excludedWorkers.size());
     Assert.assertEquals(2, masterStatusSystem.manuallyExcludedWorkers.size());
     Assert.assertEquals(3, masterStatusSystem.hostnameSet.size());
-    Assert.assertEquals(
-        conf.metricsAppTopDiskUsageWindowSize(),
-        masterStatusSystem.appDiskUsageMetric.snapShots().length);
-    Assert.assertEquals(
-        conf.metricsAppTopDiskUsageCount(),
-        masterStatusSystem.appDiskUsageMetric.currentSnapShot().get().topNItems().length);
-    Assert.assertEquals(
-        originCurrentSnapshot, masterStatusSystem.appDiskUsageMetric.currentSnapShot().get());
-    Assert.assertArrayEquals(originSnapshots, masterStatusSystem.appDiskUsageMetric.snapShots());
 
     masterStatusSystem.restoreMetaFromFile(tmpFile);
-    Assert.assertEquals(3, masterStatusSystem.workers.size());
+    Assert.assertEquals(3, masterStatusSystem.workersMap.size());
   }
 
   private String getNewReqeustId() {
