@@ -20,52 +20,23 @@ package org.apache.celeborn.service.deploy.master.http.api
 import javax.ws.rs.client.Entity
 import javax.ws.rs.core.{Form, MediaType}
 
-import com.google.common.io.Files
-
-import org.apache.celeborn.common.CelebornConf
-import org.apache.celeborn.common.util.{CelebornExitKind, ThreadUtils, Utils}
 import org.apache.celeborn.server.common.HttpService
 import org.apache.celeborn.server.common.http.ApiBaseResourceSuite
-import org.apache.celeborn.service.deploy.master.{Master, MasterArguments}
+import org.apache.celeborn.service.deploy.master.{Master, MasterClusterFeature}
 
-class ApiMasterResourceSuite extends ApiBaseResourceSuite {
+class ApiMasterResourceSuite extends ApiBaseResourceSuite with MasterClusterFeature {
   private var master: Master = _
 
   override protected def httpService: HttpService = master
 
-  def getTmpDir(): String = {
-    val tmpDir = Files.createTempDir()
-    tmpDir.deleteOnExit()
-    tmpDir.getAbsolutePath
-  }
-
   override def beforeAll(): Unit = {
-    val randomMasterPort = Utils.selectRandomPort(1024, 65535)
-    val randomHttpPort = randomMasterPort + 1
-    celebornConf.set(CelebornConf.HA_ENABLED.key, "false")
-    celebornConf.set(CelebornConf.HA_MASTER_RATIS_STORAGE_DIR.key, getTmpDir())
-    celebornConf.set(CelebornConf.WORKER_STORAGE_DIRS.key, getTmpDir())
-    celebornConf.set(CelebornConf.MASTER_HTTP_HOST.key, "127.0.0.1")
-    celebornConf.set(CelebornConf.MASTER_HTTP_PORT.key, randomHttpPort.toString)
-
-    val args = Array("-h", "localhost", "-p", randomMasterPort.toString)
-
-    val masterArgs = new MasterArguments(args, celebornConf)
-    master = new Master(celebornConf, masterArgs)
-    ThreadUtils.newThread(
-      new Runnable {
-        override def run(): Unit = {
-          master.initialize()
-        }
-      },
-      "master-init-thread").start()
+    master = setupMasterWithRandomPort(celebornConf.getAll.toMap)
     super.beforeAll()
   }
 
   override def afterAll(): Unit = {
     super.afterAll()
-    master.stop(CelebornExitKind.EXIT_IMMEDIATELY)
-    master.rpcEnv.shutdown()
+    shutdownMaster()
   }
 
   test("masterGroupInfo") {
@@ -116,4 +87,9 @@ class ApiMasterResourceSuite extends ApiBaseResourceSuite {
         .post(Entity.entity(form, MediaType.APPLICATION_FORM_URLENCODED_TYPE))
     assert(200 == response.getStatus)
   }
+
+  test("decommissionWorkers") {
+    val response = webTarget.path("decommissionWorkers").request(MediaType.TEXT_PLAIN).get()
+    assert(200 == response.getStatus)
+  }
 }
