@@ -45,6 +45,7 @@ import org.apache.celeborn.common.protocol.message.StatusCode
 import org.apache.celeborn.common.unsafe.Platform
 import org.apache.celeborn.common.util.{ExceptionUtils, Utils}
 import org.apache.celeborn.service.deploy.worker.congestcontrol.CongestionController
+import org.apache.celeborn.service.deploy.worker.memory.MemoryManager
 import org.apache.celeborn.service.deploy.worker.storage.{LocalFlusher, PartitionDataWriter, StorageManager}
 
 class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler with Logging {
@@ -64,6 +65,8 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
   private var storageManager: StorageManager = _
   private var workerPartitionSplitEnabled: Boolean = _
   private var workerReplicateRandomConnectionEnabled: Boolean = _
+  private var workerPushDataMergeBufferEnabled: Boolean = _
+  private var workerDirectMemoryRatioToMergeBuffer: Double = _
 
   private var testPushPrimaryDataTimeout: Boolean = _
   private var testPushReplicaDataTimeout: Boolean = _
@@ -83,7 +86,8 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
     shutdown = worker.shutdown
     workerPartitionSplitEnabled = worker.conf.workerPartitionSplitEnabled
     workerReplicateRandomConnectionEnabled = worker.conf.workerReplicateRandomConnectionEnabled
-
+    workerPushDataMergeBufferEnabled = worker.conf.workerPushDataMergeBufferEnabled
+    workerDirectMemoryRatioToMergeBuffer = worker.conf.workerDirectMemoryRatioToMergeBuffer
     testPushPrimaryDataTimeout = worker.conf.testPushPrimaryDataTimeout
     testPushReplicaDataTimeout = worker.conf.testPushReplicaDataTimeout
     registered = Some(worker.registered)
@@ -1020,7 +1024,6 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
         pushData.`type`(),
         isPrimary,
         pushData.requestId,
-        null,
         location,
         if (isPrimary) WorkerSource.PRIMARY_PUSH_DATA_TIME else WorkerSource.REPLICA_PUSH_DATA_TIME,
         callback)
@@ -1054,27 +1057,14 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
     val writePromise = Promise[Array[StatusCode]]()
     writeLocalData(Seq(fileWriter), body, shuffleKey, isPrimary, None, writePromise)
     // for primary, send data to replica
-    if (location.hasPeer && isPrimary) {
-      // to do
-      Try(Await.result(writePromise.future, Duration.Inf)) match {
-        case Success(result) =>
-          if (result(0) != StatusCode.SUCCESS) {
-            wrappedCallback.onFailure(new CelebornIOException("Write data failed!"))
-          } else {
-            wrappedCallback.onSuccess(ByteBuffer.wrap(Array[Byte]()))
-          }
-        case Failure(e) => wrappedCallback.onFailure(e)
-      }
-    } else {
-      Try(Await.result(writePromise.future, Duration.Inf)) match {
-        case Success(result) =>
-          if (result(0) != StatusCode.SUCCESS) {
-            wrappedCallback.onFailure(new CelebornIOException("Write data failed!"))
-          } else {
-            wrappedCallback.onSuccess(ByteBuffer.wrap(Array[Byte]()))
-          }
-        case Failure(e) => wrappedCallback.onFailure(e)
-      }
+    Try(Await.result(writePromise.future, Duration.Inf)) match {
+      case Success(result) =>
+        if (result(0) != StatusCode.SUCCESS) {
+          wrappedCallback.onFailure(new CelebornIOException("Write data failed!"))
+        } else {
+          wrappedCallback.onSuccess(ByteBuffer.wrap(Array[Byte]()))
+        }
+      case Failure(e) => wrappedCallback.onFailure(e)
     }
   }
 
@@ -1238,7 +1228,6 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
         messageType,
         isPrimary,
         requestId,
-        null,
         location,
         if (isPrimary) workerSourcePrimary else workerSourceReplica,
         callback)
@@ -1306,12 +1295,7 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
         case _ => throw new IllegalArgumentException(s"Not support $messageType yet")
       }
       // for primary , send data to replica
-      if (location.hasPeer && isPrimary) {
-        // TODO replica
-        wrappedCallback.onSuccess(ByteBuffer.wrap(Array[Byte]()))
-      } else {
-        wrappedCallback.onSuccess(ByteBuffer.wrap(Array[Byte]()))
-      }
+      wrappedCallback.onSuccess(ByteBuffer.wrap(Array[Byte]()))
     } catch {
       case t: Throwable =>
         callback.onFailure(new CelebornIOException(s"$messageType failed", t))
@@ -1322,7 +1306,6 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
       messageType: Message.Type,
       isPrimary: Boolean,
       requestId: Long,
-      softSplit: AtomicBoolean,
       location: PartitionLocation,
       workerSourceTime: String,
       callback: RpcResponseCallback)
@@ -1335,8 +1318,6 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
           resp.put(response)
           resp.flip()
           callback.onSuccess(resp)
-        } else if (softSplit != null && softSplit.get()) {
-          callback.onSuccess(ByteBuffer.wrap(Array[Byte](StatusCode.SOFT_SPLIT.getValue)))
         } else {
           callback.onSuccess(response)
         }
@@ -1407,6 +1388,7 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
     val (messagePrimary, messageReplica) =
       messageType match {
         case Type.PUSH_DATA =>
+          workerSource.incCounter(WorkerSource.WRITE_DATA_FAIL_COUNT)
           (
             StatusCode.PUSH_DATA_WRITE_FAIL_PRIMARY,
             StatusCode.PUSH_DATA_WRITE_FAIL_REPLICA)
@@ -1514,6 +1496,26 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
       hardSplitIndexes: Array[Int] = Array.empty[Int]): Unit = {
     val length = fileWriters.length
     val result = new Array[StatusCode](length)
+
+    var finalBody: ByteBuf = body
+    var copyBody: ByteBuf = null
+    if (workerPushDataMergeBufferEnabled &&
+      MemoryManager.instance().workerMemoryUsageRatio() > workerDirectMemoryRatioToMergeBuffer) {
+      val numBytes = body.readableBytes()
+      try {
+        copyBody = body.alloc.directBuffer(numBytes)
+        // this method do not increase the readerIndex of source buffer, when oom
+        // happens, we can fall back to the original buffer
+        copyBody.writeBytes(body, body.readerIndex, numBytes)
+        finalBody = copyBody
+      } catch {
+        case e: OutOfMemoryError =>
+          logError(s"caught oom when consolidate data failed, size: $numBytes", e)
+        case e: Throwable =>
+          logError(s"consolidate data failed, size: $numBytes", e)
+      }
+    }
+
     def writeData(
         fileWriter: PartitionDataWriter,
         body: ByteBuf,
@@ -1561,14 +1563,14 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
           } else {
             fileWriter = fileWriters(index)
             if (!writePromise.isCompleted) {
-              val offset = body.readerIndex() + batchOffsets(index)
+              val offset = finalBody.readerIndex() + batchOffsets(index)
               val length =
                 if (index == fileWriters.length - 1) {
-                  body.readableBytes() - batchOffsets(index)
+                  finalBody.readableBytes() - batchOffsets(index)
                 } else {
                   batchOffsets(index + 1) - batchOffsets(index)
                 }
-              val batchBody = body.slice(offset, length)
+              val batchBody = finalBody.slice(offset, length)
               writeData(fileWriter, batchBody, shuffleKey, index)
             } else {
               fileWriter.decrementPendingWrites()
@@ -1577,12 +1579,16 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
           index += 1
         }
       case _ =>
-        writeData(fileWriters.head, body, shuffleKey, 0)
+        writeData(fileWriters.head, finalBody, shuffleKey, 0)
     }
     if (!writePromise.isCompleted) {
       workerSource.incCounter(WorkerSource.WRITE_DATA_SUCCESS_COUNT)
       writePromise.success(result)
     }
+    // manually release copyBody to avoid memory leak
+    if (copyBody != null) {
+      copyBody.release()
+    }
   }
 
   private def nextValueOrElse(iterator: Iterator[Int], defaultValue: Int): Int = {
