@@ -26,10 +26,11 @@ import java.util.concurrent.atomic.{AtomicInteger, AtomicLong}
 import java.util.function.{BiConsumer, IntUnaryOperator}
 
 import scala.collection.JavaConverters._
+import scala.collection.mutable
 import scala.concurrent.duration._
 
 import com.google.common.annotations.VisibleForTesting
-import io.netty.buffer.{ByteBufAllocator, PooledByteBufAllocator}
+import io.netty.buffer.ByteBufAllocator
 import org.apache.commons.io.FileUtils
 import org.apache.hadoop.fs.{FileSystem, Path}
 import org.apache.hadoop.fs.permission.FsPermission
@@ -394,12 +395,6 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
 
   private def getNextIndex = counter.getAndUpdate(counterOperator)
 
-  private val newMapFunc =
-    new java.util.function.Function[String, ConcurrentHashMap[String, DiskFileInfo]]() {
-      override def apply(key: String): ConcurrentHashMap[String, DiskFileInfo] =
-        JavaUtils.newConcurrentHashMap()
-    }
-
   private val diskFileInfoMapFunc =
     new java.util.function.Function[String, ConcurrentHashMap[String, DiskFileInfo]]() {
       override def apply(key: String): ConcurrentHashMap[String, DiskFileInfo] =
@@ -644,16 +639,12 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
       logInfo(s"Cleanup expired shuffle $shuffleKey.")
       if (diskFileInfos.containsKey(shuffleKey)) {
         val removedFileInfos = diskFileInfos.remove(shuffleKey)
-        var isDfsExpired = false
-        var isHdfs = false
-        var isOss = false
+        val expireStorageTypes = mutable.Set[StorageInfo.Type]()
         if (removedFileInfos != null) {
           removedFileInfos.asScala.foreach {
             case (_, fileInfo) =>
               if (cleanFileInternal(shuffleKey, fileInfo)) {
-                isDfsExpired = true
-                isHdfs = fileInfo.isHdfs
-                isOss = fileInfo.isOSS
+                expireStorageTypes.add(fileInfo.getStorageType)
               }
           }
         }
@@ -666,23 +657,19 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
             deleteDirectory(file, diskOperators.get(diskInfo.mountPoint))
           }
         }
-        if (isDfsExpired) {
+        expireStorageTypes.foreach(storageType => {
           try {
             val dir =
-              if (hasHDFSStorage && isHdfs) hdfsDir
-              else if (hasOssStorage && isOss) ossDir
+              if (storageType == StorageInfo.Type.HDFS) hdfsDir
+              else if (storageType == StorageInfo.Type.OSS) ossDir
               else s3Dir
-            val storageInfo =
-              if (hasHDFSStorage && isHdfs) StorageInfo.Type.HDFS
-              else if (hasOssStorage && isOss) StorageInfo.Type.OSS
-              else StorageInfo.Type.S3
-            StorageManager.hadoopFs.get(storageInfo).delete(
+            StorageManager.hadoopFs.get(storageType).delete(
               new Path(new Path(dir, conf.workerWorkingDir), s"$appId/$shuffleId"),
               true)
           } catch {
-            case e: Exception => logWarning("Clean expired DFS shuffle failed.", e)
+            case e: Exception => logWarning(s"Clean expired $storageType shuffle failed.", e)
           }
-        }
+        })
         if (workerGracefulShutdown) {
           committedFileInfos.remove(shuffleKey)
           if (cleanDB) {
@@ -779,21 +766,26 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
               false
           }
         }
-
-      val dfsCleaned = hadoopFs match {
-        case dfs: FileSystem =>
-          val dfsDir = if (hasHDFSStorage) hdfsDir else if (hasOssStorage) ossDir else s3Dir
-          val dfsWorkPath = new Path(dfsDir, conf.workerWorkingDir)
-          // DFS path not exist when first time initialize
-          if (dfs.exists(dfsWorkPath)) {
-            !dfs.listFiles(dfsWorkPath, false).hasNext
-          } else {
-            true
+      val dfsCleaned = hadoopFs == null || hadoopFs.asScala.forall {
+        case (storageType, fs) =>
+          val dfsDir =
+            if (storageType == StorageInfo.Type.HDFS)
+              hdfsDir
+            else if (storageType == StorageInfo.Type.OSS) ossDir
+            else s3Dir
+          try {
+            val dfsWorkPath = new Path(dfsDir, conf.workerWorkingDir)
+            // DFS path not exist when first time initialize
+            !fs.exists(dfsWorkPath) || !fs.listFiles(dfsWorkPath, false).hasNext
+          } catch {
+            case t: Throwable =>
+              // When DFS is not accessible, assume it's cleaned
+              logError("checkIfWorkingDirCleaned failed.", t)
+              true
           }
         case _ =>
           true
       }
-
       if (localCleaned && dfsCleaned) {
         return true
       }
@@ -909,7 +901,7 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
   def updateDiskInfos(): Unit = this.synchronized {
     disksSnapshot()
       .filter(diskInfo =>
-        diskInfo.status != DiskStatus.IO_HANG || diskInfo.status != DiskStatus.READ_OR_WRITE_FAILURE)
+        diskInfo.status != DiskStatus.IO_HANG && diskInfo.status != DiskStatus.READ_OR_WRITE_FAILURE)
       .foreach {
         diskInfo =>
           val totalUsage = diskInfo.dirs.map { dir =>
@@ -1000,7 +992,7 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
       shuffleKey: String,
       fileName: String,
       fileInfo: DiskFileInfo): Unit = {
-    committedFileInfos.computeIfAbsent(shuffleKey, newMapFunc).put(fileName, fileInfo)
+    committedFileInfos.computeIfAbsent(shuffleKey, diskFileInfoMapFunc).put(fileName, fileInfo)
   }
 
   def getActiveShuffleSize: Long = {
