@@ -18,7 +18,6 @@
 package org.apache.celeborn.service.deploy.worker;
 
 import static org.junit.Assert.*;
-import static org.mockito.ArgumentMatchers.anyString;
 import static org.mockito.Mockito.*;
 
 import java.io.File;
@@ -45,6 +44,7 @@ import org.slf4j.LoggerFactory;
 
 import org.apache.celeborn.common.CelebornConf;
 import org.apache.celeborn.common.identity.UserIdentifier;
+import org.apache.celeborn.common.meta.DiskFileInfo;
 import org.apache.celeborn.common.meta.FileInfo;
 import org.apache.celeborn.common.network.buffer.NioManagedBuffer;
 import org.apache.celeborn.common.network.client.RpcResponseCallback;
@@ -78,7 +78,7 @@ public class FetchHandlerSuiteJ {
 
   private static final Logger LOG = LoggerFactory.getLogger(FetchHandlerSuiteJ.class);
 
-  private static CelebornConf conf = new CelebornConf();
+  private static final CelebornConf conf = new CelebornConf();
 
   private static final Random random = new Random();
 
@@ -91,7 +91,7 @@ public class FetchHandlerSuiteJ {
     byte[] batchHeader = new byte[16];
     File shuffleFile = File.createTempFile("celeborn", UUID.randomUUID().toString());
 
-    FileInfo fileInfo = new FileInfo(shuffleFile, userIdentifier);
+    DiskFileInfo fileInfo = new DiskFileInfo(shuffleFile, userIdentifier, conf);
     FileOutputStream fileOutputStream = new FileOutputStream(shuffleFile);
     FileChannel channel = fileOutputStream.getChannel();
     Map<Integer, Integer> batchIds = new HashMap<>();
@@ -130,7 +130,7 @@ public class FetchHandlerSuiteJ {
     for (long offset = conf.shuffleChunkSize();
         offset <= originFileLen;
         offset += conf.shuffleChunkSize()) {
-      fileInfo.getChunkOffsets().add(offset);
+      (fileInfo.getReduceFileMeta()).addChunkOffset(offset);
     }
     // update sorted fileInfo chunk offsets
     fileInfo.updateBytesFlushed(originFileLen);
@@ -140,11 +140,13 @@ public class FetchHandlerSuiteJ {
   public void cleanup(FileInfo fileInfo) throws IOException {
     if (fileInfo != null) {
       // origin file
-      JavaUtils.deleteRecursively(fileInfo.getFile());
+      JavaUtils.deleteRecursively(((DiskFileInfo) fileInfo).getFile());
       // sorted file
-      JavaUtils.deleteRecursively(new File(fileInfo.getFile().getPath() + ".sorted"));
+      JavaUtils.deleteRecursively(
+          new File(((DiskFileInfo) fileInfo).getFile().getPath() + ".sorted"));
       // index file
-      JavaUtils.deleteRecursively(new File(fileInfo.getFile().getPath() + ".index"));
+      JavaUtils.deleteRecursively(
+          new File(((DiskFileInfo) fileInfo).getFile().getPath() + ".index"));
     }
   }
 
@@ -319,7 +321,7 @@ public class FetchHandlerSuiteJ {
     return fetchHandler;
   }
 
-  private final String shuffleKey = "dummyShuffleKey";
+  private final String shuffleKey = "dummyShuffleKey-123";
   private final String fileName = "dummyFileName";
   private final long dummyRequestId = 0;
 
@@ -439,12 +441,12 @@ public class FetchHandlerSuiteJ {
   }
 
   private void checkOriginFileBeDeleted(FileInfo fileInfo) {
-    assertTrue(!fileInfo.getFilePath().endsWith(".sorted"));
+    assertFalse(((DiskFileInfo) fileInfo).getFilePath().endsWith(".sorted"));
     long startTs = System.currentTimeMillis();
     boolean deleted = false;
     long timeout = 5 * 1000; // 5s
     while (!deleted) {
-      deleted = !fileInfo.getFile().exists();
+      deleted = !((DiskFileInfo) fileInfo).getFile().exists();
       if (System.currentTimeMillis() - startTs > timeout) {
         fail("Origin file was not deleted within the expected timeout of 5 seconds.");
       }
