@@ -56,7 +56,7 @@ class StoragePolicyCase2 extends CelebornFunSuite {
   when(mockedStorageManager.storageBufferAllocator).thenAnswer(UnpooledByteBufAllocator.DEFAULT)
 
   val mockedDiskFile = mock[DiskFileInfo]
-  val mockedFlusher = mock[Flusher]
+  val mockedFlusher = mock[LocalFlusher]
   val mockedFile = mock[File]
   when(
     mockedStorageManager.createDiskFile(
@@ -101,6 +101,7 @@ class StoragePolicyCase2 extends CelebornFunSuite {
     when(mockedPartitionWriterContext.getPartitionLocation).thenAnswer(localHintPartitionLocatioin)
     when(mockedPartitionWriterContext.getPartitionType).thenAnswer(PartitionType.REDUCE)
     when(mockedStorageManager.localOrDfsStorageAvailable).thenAnswer(true)
+    when(mockedDiskFile.getStorageType).thenAnswer(StorageInfo.Type.HDD)
     val conf = new CelebornConf()
     conf.set("celeborn.worker.storage.storagePolicy.createFilePolicy", "SSD,HDD,HDFS,OSS,S3")
     val storagePolicy = new StoragePolicy(conf, mockedStorageManager, mockedSource)
